import { filter } from 'rxjs/operators';
/**
 * @public
 */
var WsRpc = /** @class */ (function () {
    function WsRpc(subject, getRequestId, getError, timeout) {
        if (timeout === void 0) { timeout = 10000; }
        this.subject = subject;
        this.getRequestId = getRequestId;
        this.getError = getError;
        this.timeout = timeout;
        this.lastRequestId = 0;
    }
    WsRpc.prototype.send = function (send, timeout) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var requestId = _this.generateRequestId();
            var timeoutId;
            var subscription = _this.subject
                .pipe(filter(function (r) { return _this.getRequestId(r) === requestId; }))
                .subscribe(function (r) {
                if (timeoutId) {
                    clearTimeout(timeoutId);
                }
                subscription.unsubscribe();
                var error = _this.getError(r);
                if (error) {
                    reject(new Error(error));
                }
                else {
                    resolve(r);
                }
            });
            timeoutId = setTimeout(function () {
                subscription.unsubscribe();
                reject(new Error('timeout'));
            }, timeout || _this.timeout);
            send(requestId);
        });
    };
    WsRpc.prototype.generateRequestId = function () {
        this.lastRequestId = this.lastRequestId < 4294967295 ? this.lastRequestId + 1 : 1; // 4294967295 = 2^32 - 1
        return this.lastRequestId;
    };
    return WsRpc;
}());
export default WsRpc;
