"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const operators_1 = require("rxjs/operators");
/**
 * @public
 */
class WsRpc {
    constructor(subject, getRequestId, getError, timeout = 10000) {
        this.subject = subject;
        this.getRequestId = getRequestId;
        this.getError = getError;
        this.timeout = timeout;
        this.lastRequestId = 0;
    }
    send(send, timeout) {
        return new Promise((resolve, reject) => {
            const requestId = this.generateRequestId();
            let timeoutId;
            const subscription = this.subject
                .pipe(operators_1.filter((r) => this.getRequestId(r) === requestId))
                .subscribe((r) => {
                if (timeoutId) {
                    clearTimeout(timeoutId);
                }
                subscription.unsubscribe();
                const error = this.getError(r);
                if (error) {
                    reject(new Error(error));
                }
                else {
                    resolve(r);
                }
            });
            timeoutId = setTimeout(() => {
                subscription.unsubscribe();
                reject(new Error('timeout'));
            }, timeout || this.timeout);
            send(requestId);
        });
    }
    generateRequestId() {
        this.lastRequestId = this.lastRequestId < 4294967295 ? this.lastRequestId + 1 : 1; // 4294967295 = 2^32 - 1
        return this.lastRequestId;
    }
}
exports.default = WsRpc;
